@@ -1,4 +1,4 @@
-set rcsid {$Id: capi3ref.tcl,v 1.24 2005/07/22 22:53:16 drh Exp $}
+set rcsid {$Id: capi3ref.tcl,v 1.45 2006/09/15 16:58:49 drh Exp $}
 source common.tcl
 header {C/C++ Interface For SQLite Version 3}
 puts {
@@ -6,15 +6,59 @@ puts {
 }
 
 proc api {name prototype desc {notused x}} {
-  global apilist
+  global apilist specialname
   if {$name==""} {
     regsub -all {sqlite3_[a-z0-9_]+\(} $prototype \
       {[lappend name [string trimright & (]]} x1
     subst $x1
+  } else {
+    lappend specialname $name
   }
   lappend apilist [list $name $prototype $desc]
 }
 
+api {extended-result-codes} {
+#define SQLITE_IOERR_READ       
+#define SQLITE_IOERR_SHORT_READ 
+#define SQLITE_IOERR_WRITE      
+#define SQLITE_IOERR_FSYNC      
+#define SQLITE_IOERR_DIR_FSYNC  
+#define SQLITE_IOERR_TRUNCATE   
+#define SQLITE_IOERR_FSTAT      
+#define SQLITE_IOERR_UNLOCK     
+#define SQLITE_IOERR_RDLOCK     
+...
+} {
+In its default configuration, SQLite API routines return one of 26 integer
+result codes described at result-codes.  However, experience has shown that
+many of these result codes are too course-grained.  They do not provide as
+much information about problems as users might like.  In an effort to
+address this, newer versions of SQLite (version 3.3.8 and later) include
+support for additional result codes that provide more detailed information
+about errors.  The extended result codes are enabled (or disabled) for 
+each database
+connection using the sqlite3_extended_result_codes() API.
+
+Some of the available extended result codes are listed above.
+We expect the number of extended result codes will be expand
+over time.  Software that uses extended result codes should expect
+to see new result codes in future releases of SQLite.
+
+The symbolic name for an extended result code always contains a related
+primary result code as a prefix.  Primary result codes contain a single
+"_" character.  Extended result codes contain two or more "_" characters.
+The numeric value of an extended result code can be converted to its
+corresponding primary result code by masking off the lower 8 bytes.
+
+A complete list of available extended result codes and
+details about the meaning of the various extended result codes can be
+found by consulting the C code, especially the sqlite3.h header
+file and its antecedent sqlite.h.in.  Additional information
+is also available at the SQLite wiki:
+http://www.sqlite.org/cvstrac/wiki?p=ExtendedResultCodes
+}
+
+
 api {result-codes} {
 #define SQLITE_OK           0   /* Successful result */
 #define SQLITE_ERROR        1   /* SQL error or missing database */
@@ -45,6 +89,27 @@ api {result-codes} {
 } {
 Many SQLite functions return an integer result code from the set shown
 above in order to indicates success or failure.
+
+The result codes above are the only ones returned by SQLite in its
+default configuration.  However, the sqlite3_extended_result_codes()
+API can be used to set a database connectoin to return more detailed
+result codes.  See the documentation on sqlite3_extended_result_codes()
+or extended-result-codes for additional information.
+}
+
+api {} {
+  int sqlite3_extended_result_codes(sqlite3*, int onoff);
+} {
+This routine enables or disabled extended-result-codes feature.
+By default, SQLite API routines return one of only 26 integer
+result codes described at result-codes.  When extended result codes
+are enabled by this routine, the repetoire of result codes can be
+much larger and can (hopefully) provide more detailed information
+about the cause of an error.
+
+The second argument is a boolean value that turns extended result
+codes on and off.  Extended result codes are off by default for
+backwards compatibility with older versions of SQLite.
 }
 
 api {} {
@@ -66,15 +131,19 @@ api {} {
   same aggregate instance) the same buffer is returned.  The implementation
   of the aggregate can use the returned buffer to accumulate data.
 
-  The buffer allocated is freed automatically by SQLite.
+  The buffer is freed automatically by SQLite when the query that
+  invoked the aggregate function terminates.
 }
 
 api {} {
   int sqlite3_aggregate_count(sqlite3_context*);
 } {
-  The next routine returns the number of calls to xStep for a particular
-  aggregate function instance.  The current call to xStep counts so this
-  routine always returns at least 1.
+  This function is deprecated.   It continues to exist so as not to
+  break any legacy code that might happen to use it.  But it should not
+  be used in any new code.
+
+  In order to encourage people to not use this function, we are not going
+  to tell you what it does.
 }
 
 api {} {
@@ -89,7 +158,8 @@ api {} {
   #define SQLITE_TRANSIENT   ((void(*)(void *))-1)
 } {
  In the SQL strings input to sqlite3_prepare() and sqlite3_prepare16(),
- one or more literals can be replace by a parameter "?" or ":AAA" or "\$VVV"
+ one or more literals can be replace by a parameter "?" or ":AAA" or 
+ "@AAA" or "\$VVV"
  where AAA is an alphanumeric identifier and VVV is a variable name according
  to the syntax rules of the TCL programming language.
  The values of these parameters (also called "host parameter names")
@@ -108,11 +178,11 @@ api {} {
 
  In those
  routines that have a fourth argument, its value is the number of bytes
- in the parameter.  This is the number of characters for UTF-8 strings
- and the number of bytes for UTF-16 strings and blobs.  The number
+ in the parameter.  To be clear: the value is the number of bytes in the
+ string, not the number of characters.  The number
  of bytes does not include the zero-terminator at the end of strings.
  If the fourth parameter is negative, the length of the string is
- computed using strlen().
+ number of bytes up to the first zero terminator.
 
  The fifth argument to sqlite3_bind_blob(), sqlite3_bind_text(), and
  sqlite3_bind_text16() is a destructor used to dispose of the BLOB or
@@ -120,12 +190,19 @@ api {} {
  special value SQLITE_STATIC, then the library assumes that the information
  is in static, unmanaged space and does not need to be freed.  If the
  fifth argument has the value SQLITE_TRANSIENT, then SQLite makes its
- own private copy of the data before returning.
+ own private copy of the data immediately, before the sqlite3_bind_*()
+ routine returns.
 
  The sqlite3_bind_*() routines must be called after
  sqlite3_prepare() or sqlite3_reset() and before sqlite3_step().
  Bindings are not cleared by the sqlite3_reset() routine.
  Unbound parameters are interpreted as NULL.
+
+ These routines return SQLITE_OK on success or an error code if
+ anything goes wrong.  SQLITE_RANGE is returned if the parameter
+ index is out of range.  SQLITE_NOMEM is returned if malloc fails.
+ SQLITE_MISUSE is returned if these routines are called on a virtual
+ machine that is the wrong state or which has already been finalized.
 }
 
 api {} {
@@ -139,11 +216,13 @@ api {} {
   const char *sqlite3_bind_parameter_name(sqlite3_stmt*, int n);
 } {
   Return the name of the n-th parameter in the precompiled statement.
-  Parameters of the form ":AAA" or "\$VVV" have a name which is the
-  string ":AAA" or "\$VVV".  In other words, the initial ":" or "$"
+  Parameters of the form ":AAA" or "@AAA" or "\$VVV" have a name which is the
+  string ":AAA" or "\$VVV".  In other words, the initial ":" or "$" or "@"
   is included as part of the name.
   Parameters of the form "?" have no name.
 
+  The first bound parameter has an index of 1, not 0.
+
   If the value n is out of range or if the n-th parameter is nameless,
   then NULL is returned.  The returned string is always in the
   UTF-8 encoding.
@@ -167,7 +246,7 @@ api {} {
  If the busy callback is NULL, then SQLITE_BUSY is returned immediately
  upon encountering the lock.
  If the busy callback is not NULL, then the
- callback might invoked with two arguments.  The
+ callback will be invoked with two arguments.  The
  second argument is the number of prior calls to the busy callback
  for the same lock.  If the
  busy callback returns 0, then no additional attempts are made to
@@ -198,6 +277,11 @@ api {} {
  database.  Closing the database from a busy handler will delete 
  data structures out from under the executing query and will 
  probably result in a coredump.
+
+ There can only be a single busy handler defined for each database
+ connection.  Setting a new busy handler clears any previous one.
+ Note that calling sqlite3_busy_timeout() will also set or clear
+ the busy handler.
 }
 
 api {} {
@@ -211,6 +295,11 @@ api {} {
 
  Calling this routine with an argument less than or equal to zero
  turns off all busy handlers.
+
+ There can only be a single busy handler for a particular database
+ connection.  If another busy handler was defined  
+ (using sqlite3_busy_handler()) prior to calling
+ this routine, that other busy handler is cleared.
 }
 
 api {} {
@@ -357,11 +446,10 @@ const void *sqlite3_column_decltype16(sqlite3_stmt*,int);
  The first argument is a prepared SQL statement. If this statement
  is a SELECT statement, the Nth column of the returned result set 
  of the SELECT is a table column then the declared type of the table
- column is returned. If the Nth column of the result set is not at table
+ column is returned. If the Nth column of the result set is not a table
  column, then a NULL pointer is returned. The returned string is 
  UTF-8 encoded for sqlite3_column_decltype() and UTF-16 encoded
- for sqlite3_column_decltype16().
- For example, in the database schema:
+ for sqlite3_column_decltype16(). For example, in the database schema:
 
  <blockquote><pre>
  CREATE TABLE t1(c1 INTEGER);
@@ -370,12 +458,140 @@ const void *sqlite3_column_decltype16(sqlite3_stmt*,int);
  And the following statement compiled:
 
  <blockquote><pre>
- SELECT c1 + 1, 0 FROM t1;
+ SELECT c1 + 1, c1 FROM t1;
  </pre></blockquote>
 
  Then this routine would return the string "INTEGER" for the second
  result column (i==1), and a NULL pointer for the first result column
  (i==0).
+
+ If the following statements were compiled then this routine would
+ return "INTEGER" for the first (only) result column.
+
+ <blockquote><pre>
+ SELECT (SELECT c1) FROM t1;
+ SELECT (SELECT c1 FROM t1);
+ SELECT c1 FROM (SELECT c1 FROM t1);
+ SELECT * FROM (SELECT c1 FROM t1);
+ SELECT * FROM (SELECT * FROM t1);
+ </pre></blockquote>
+}
+
+api {} {
+  int sqlite3_table_column_metadata(
+    sqlite3 *db,                /* Connection handle */
+    const char *zDbName,        /* Database name or NULL */
+    const char *zTableName,     /* Table name */
+    const char *zColumnName,    /* Column name */
+    char const **pzDataType,    /* OUTPUT: Declared data type */
+    char const **pzCollSeq,     /* OUTPUT: Collation sequence name */
+    int *pNotNull,              /* OUTPUT: True if NOT NULL constraint exists */
+    int *pPrimaryKey,           /* OUTPUT: True if column part of PK */
+    int *pAutoinc               /* OUTPUT: True if colums is auto-increment */
+  );
+} {
+ This routine is used to obtain meta information about a specific column of a
+ specific database table accessible using the connection handle passed as the
+ first function argument.
+
+ The column is identified by the second, third and fourth parameters to 
+ this function. The second parameter is either the name of the database
+ (i.e. "main", "temp" or an attached database) containing the specified
+ table or NULL. If it is NULL, then all attached databases are searched
+ for the table using the same algorithm as the database engine uses to 
+ resolve unqualified table references.
+
+ The third and fourth parameters to this function are the table and column 
+ name of the desired column, respectively. Neither of these parameters 
+ may be NULL.
+
+ Meta information is returned by writing to the memory locations passed as
+ the 5th and subsequent parameters to this function. Any of these 
+ arguments may be NULL, in which case the corresponding element of meta 
+ information is ommitted.
+
+<pre>
+ Parameter     Output Type      Description
+ -----------------------------------
+   5th         const char*      Declared data type 
+   6th         const char*      Name of the columns default collation sequence 
+   7th         int              True if the column has a NOT NULL constraint
+   8th         int              True if the column is part of the PRIMARY KEY
+   9th         int              True if the column is AUTOINCREMENT
+</pre>
+
+ The memory pointed to by the character pointers returned for the 
+ declaration type and collation sequence is valid only until the next 
+ call to any sqlite API function.
+
+ This function may load one or more schemas from database files. If an
+ error occurs during this process, or if the requested table or column
+ cannot be found, an SQLITE error code is returned and an error message
+ left in the database handle (to be retrieved using sqlite3_errmsg()).
+ Specifying an SQL view instead of a table as the third argument is also
+ considered an error.
+
+ If the specified column is "rowid", "oid" or "_rowid_" and an 
+ INTEGER PRIMARY KEY column has been explicitly declared, then the output 
+ parameters are set for the explicitly declared column. If there is no
+ explicitly declared IPK column, then the data-type is "INTEGER", the
+ collation sequence "BINARY" and the primary-key flag is set. Both
+ the not-null and auto-increment flags are clear.
+
+ This API is only available if the library was compiled with the
+ SQLITE_ENABLE_COLUMN_METADATA preprocessor symbol defined.
+}
+
+api {} {
+const char *sqlite3_column_database_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_database_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference,
+these functions may be used to access the name of the database (either 
+"main", "temp" or the name of an attached database) that contains
+the column. If the Nth column is not a column reference, NULL is
+returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_database_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_database_name16() returns a pointer
+to a UTF-16 encoded string. 
+}
+
+api {} {
+const char *sqlite3_column_origin_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_origin_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference,
+these functions may be used to access the schema name of the referenced 
+column in the database schema. If the Nth column is not a column 
+reference, NULL is returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_origin_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_origin_name16() returns a pointer
+to a UTF-16 encoded string. 
+}
+
+api {} {
+const char *sqlite3_column_table_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_table_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference, 
+these functions may be used to access the name of the table that 
+contains the column.  If the Nth column is not a column reference, 
+NULL is returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_table_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_table_name16() returns a pointer
+to a UTF-16 encoded string. 
 }
 
 api {} {
@@ -413,6 +629,11 @@ int sqlite3_complete16(const void *sql);
  one or more complete SQL statements.
  The argument must be a nul-terminated UTF-8 string for sqlite3_complete()
  and a nul-terminated UTF-16 string for sqlite3_complete16().
+
+ These routines do not check to see if the SQL statement is well-formed.
+ They only check to see that the statement is terminated by a semicolon
+ that is not part of a string literal and is not inside
+ the body of a trigger.
 } {}
 
 api {} {
@@ -533,7 +754,11 @@ int sqlite3_create_function16(
  only difference between these two routines is that the second argument, the
  name of the (scalar) function or aggregate, is encoded in UTF-8 for
  sqlite3_create_function() and UTF-16 for sqlite3_create_function16().
-
+ The length of the name is limited to 255 bytes, exclusive of the 
+ zero-terminator.  Note that the name length limit is in bytes, not
+ characters.  Any attempt to create a function with a longer name
+ will result in an SQLITE_ERROR error.
+ 
  The first argument is the database handle that the new function or
  aggregate is to be added to. If a single program uses more than one
  database handle internally, then user functions or aggregates must 
@@ -542,7 +767,9 @@ int sqlite3_create_function16(
 
  The third argument is the number of arguments that the function or
  aggregate takes. If this argument is -1 then the function or
- aggregate may take any number of arguments.
+ aggregate may take any number of arguments.  The maximum number
+ of arguments to a new SQL function is 127.  A number larger than
+ 127 for the third argument results in an SQLITE_ERROR error.
 
  The fourth argument, eTextRep, specifies what type of text arguments
  this function prefers to receive.  Any function should be able to work
@@ -684,10 +911,36 @@ int sqlite3_finalize(sqlite3_stmt *pStmt);
 }
 
 api {} {
-void sqlite3_free(char *z);
+void *sqlite3_malloc(int);
+void *sqlite3_realloc(void*, int);
+void sqlite3_free(void*);
 } {
- Use this routine to free memory obtained from 
- sqlite3_mprintf() or sqlite3_vmprintf().
+ These routines provide access to the memory allocator used by SQLite.
+ Depending on how SQLite has been compiled and the OS-layer backend,
+ the memory allocator used by SQLite might be the standard system
+ malloc()/realloc()/free(), or it might be something different.  With
+ certain compile-time flags, SQLite will add wrapper logic around the
+ memory allocator to add memory leak and buffer overrun detection.  The
+ OS layer might substitute a completely different memory allocator.
+ Use these APIs to be sure you are always using the correct memory
+ allocator.
+ 
+ The sqlite3_free() API, not the standard free() from the system library,
+ should always be used to free the memory buffer returned by
+ sqlite3_mprintf() or sqlite3_vmprintf() and to free the error message
+ string returned by sqlite3_exec().  Using free() instead of sqlite3_free()
+ might accidentally work on some systems and build configurations but 
+ will fail on others.
+
+ Compatibility Note:  Prior to version 3.4.0, the sqlite3_free API
+ was prototyped to take a <tt>char*</tt> parameter rather than 
+ <tt>void*</tt>.  Like this:
+<blockquote><pre>
+void sqlite3_free(char*);
+</pre></blockquote>
+ The change to using <tt>void*</tt> might cause warnings when 
+ compiling older code against
+ newer libraries, but everything should still work correctly.
 }
 
 api {} {
@@ -759,13 +1012,23 @@ api {sqlite3_interrupt} {
 api {} {
 long long int sqlite3_last_insert_rowid(sqlite3*);
 } {
- Each entry in an SQLite table has a unique integer key.  (The key is
- the value of the INTEGER PRIMARY KEY column if there is such a column,
- otherwise the key is generated at random.  The unique key is always
- available as the ROWID, OID, or _ROWID_ column.)  This routine
- returns the integer key of the most recent insert in the database.
+ Each entry in an SQLite table has a unique integer key called the "rowid".
+ The rowid is always available as an undeclared column
+ named ROWID, OID, or _ROWID_.
+ If the table has a column of type INTEGER PRIMARY KEY then that column
+ is another an alias for the rowid.
 
- This function is similar to the mysql_insert_id() function from MySQL.
+ This routine
+ returns the rowid of the most recent INSERT into the database
+ from the database connection given in the first argument.  If
+ no inserts have ever occurred on this database connection, zero
+ is returned.
+
+ If an INSERT occurs within a trigger, then the rowid of the
+ inserted row is returned by this routine as long as the trigger
+ is running.  But once the trigger terminates, the value returned
+ by this routine reverts to the last value inserted before the
+ trigger fired.
 } {}
 
 api {} {
@@ -852,6 +1115,11 @@ int sqlite3_open16(
  pass the resulting database handle off to another thread to use.  This
  restriction is due to goofy design decisions (bugs?) in the way some
  threading implementations interact with file locks.
+
+ Note to windows users:  The encoding used for the filename argument
+ of sqlite3_open() must be UTF-8, not whatever codepage is currently
+ defined.  Filenames containing international characters must be converted
+ to UTF-8 prior to passing them into sqlite3_open().
 }
 
 api {} {
@@ -992,29 +1260,55 @@ int sqlite3_set_authorizer(
 #define SQLITE_UPDATE               23   /* Table Name      Column Name     */
 #define SQLITE_ATTACH               24   /* Filename        NULL            */
 #define SQLITE_DETACH               25   /* Database Name   NULL            */
+#define SQLITE_ALTER_TABLE          26   /* Database Name   Table Name      */
+#define SQLITE_REINDEX              27   /* Index Name      NULL            */
+#define SQLITE_ANALYZE              28   /* Table Name      NULL            */
+#define SQLITE_CREATE_VTABLE        29   /* Table Name      Module Name     */
+#define SQLITE_DROP_VTABLE          30   /* Table Name      Module Name     */
+#define SQLITE_FUNCTION             31   /* Function Name   NULL            */
 
 #define SQLITE_DENY   1   /* Abort the SQL statement with an error */
 #define SQLITE_IGNORE 2   /* Don't allow access, but don't generate an error */
 } {
  This routine registers a callback with the SQLite library.  The
- callback is invoked (at compile-time, not at run-time) for each
- attempt to access a column of a table in the database.  The callback should
+ callback is invoked by sqlite3_prepare() to authorize various
+ operations against the database.  The callback should
  return SQLITE_OK if access is allowed, SQLITE_DENY if the entire
  SQL statement should be aborted with an error and SQLITE_IGNORE
- if the column should be treated as a NULL value.
+ if the operation should be treated as a no-op.
+
+ Each database connection have at most one authorizer registered
+ at a time one time.  Each call
+ to sqlite3_set_authorizer() overrides the previous authorizer.
+ Setting the callback to NULL disables the authorizer.
 
  The second argument to the access authorization function will be one
  of the defined constants shown.  These values signify what kind of operation
  is to be authorized.  The 3rd and 4th arguments to the authorization
- function will be arguments or NULL depending on which of the following
- codes is used as the second argument.  The 5th argument is the name
- of the database ("main", "temp", etc.) if applicable.  The 6th argument
+ function will be arguments or NULL depending on which of the 
+ codes is used as the second argument.  For example, if the the
+ 2nd argument code is SQLITE_READ then the 3rd argument will be the name
+ of the table that is being read from and the 4th argument will be the
+ name of the column that is being read from.  Or if the 2nd argument
+ is SQLITE_FUNCTION then the 3rd argument will be the name of the
+ function that is being invoked and the 4th argument will be NULL.
+
+ The 5th argument is the name
+ of the database ("main", "temp", etc.) where applicable.  The 6th argument
  is the name of the inner-most trigger or view that is responsible for
  the access attempt or NULL if this access attempt is directly from 
  input SQL code.
 
  The return value of the authorization function should be one of the
- constants SQLITE_OK, SQLITE_DENY, or SQLITE_IGNORE.
+ constants SQLITE_OK, SQLITE_DENY, or SQLITE_IGNORE.  A return of
+ SQLITE_OK means that the operation is permitted and that 
+ sqlite3_prepare() can proceed as normal.
+ A return of SQLITE_DENY means that the sqlite3_prepare()
+ should fail with an error.  A return of SQLITE_IGNORE causes the 
+ sqlite3_prepare() to continue as normal but the requested 
+ operation is silently converted into a no-op.  A return of SQLITE_IGNORE
+ in response to an SQLITE_READ or SQLITE_FUNCTION causes the column
+ being read or the function being invoked to return a NULL.
 
  The intent of this routine is to allow applications to safely execute
  user-entered SQL.  An appropriate callback can deny the user-entered
@@ -1050,12 +1344,46 @@ int sqlite3_step(sqlite3_stmt*);
  SQLITE_ERROR means that a run-time error (such as a constraint
  violation) has occurred.  sqlite3_step() should not be called again on
  the VM. More information may be found by calling sqlite3_errmsg().
+ A more specific error code (example: SQLITE_INTERRUPT, SQLITE_SCHEMA,
+ SQLITE_CORRUPT, and so forth) can be obtained by calling
+ sqlite3_reset() on the prepared statement.
 
  SQLITE_MISUSE means that the this routine was called inappropriately.
  Perhaps it was called on a virtual machine that had already been
  finalized or on one that had previously returned SQLITE_ERROR or
  SQLITE_DONE.  Or it could be the case that a database connection
  is being used by a different thread than the one it was created it.
+
+ <b>Goofy Interface Alert:</b>
+ The sqlite3_step() API always returns a generic error code,
+ SQLITE_ERROR, following any error other than SQLITE_BUSY and SQLITE_MISUSE.
+ You must call sqlite3_reset() (or sqlite3_finalize()) in order to find
+ the specific error code that better describes the error.  We admit that
+ this is a goofy design.  Sqlite3_step() would be much easier to use if
+ it returned the specific error code directly.  But we cannot change that
+ now without breaking backwards compatibility.
+
+ Note that there is never any harm in calling sqlite3_reset() after
+ getting back an SQLITE_ERROR from sqlite3_step().  Any API that can
+ be used after an sqlite3_step() can also be used after sqlite3_reset().
+ You may want to create a simple wrapper around sqlite3_step() to make
+ this easier.  For example:
+
+ <blockquote><pre>
+    int less_goofy_sqlite3_step(sqlite3_stmt *pStatement){
+      int rc;
+      rc = sqlite3_step(pStatement);
+      if( rc==SQLITE_ERROR ){
+        rc = sqlite3_reset(pStatement);
+      }
+      return rc;
+    }
+ </pre></blockquote>
+
+ Simply substitute the less_goofy_sqlite3_step() call above for 
+ the normal sqlite3_step() everywhere in your code, and you will
+ always get back the specific error code rather than a generic
+ SQLITE_ERROR error code.
 }
 
 api {} {
@@ -1138,23 +1466,10 @@ api {} {
 api {} {
   int sqlite3_global_recover();
 } {
- This function is called to recover from a malloc() failure that occured
- within the SQLite library. Normally, after a single malloc() fails the 
- library refuses to function (all major calls return SQLITE_NOMEM).
- This function restores the library state so that it can be used again.
-
- All existing statements (sqlite3_stmt pointers) must be finalized or
- reset before this call is made. Otherwise, SQLITE_BUSY is returned.
- If any in-memory databases are in use, either as a main or TEMP
- database, SQLITE_ERROR is returned. In either of these cases, the 
- library is not reset and remains unusable.
-
- This function is *not* threadsafe. Calling this from within a threaded
- application when threads other than the caller have used SQLite is
- dangerous and will almost certainly result in malfunctions.
-
- This functionality can be omitted from a build by defining the 
- SQLITE_OMIT_GLOBALRECOVER at compile time.
+ This function used to be involved in recovering from out-of-memory
+ errors.  But as of SQLite version 3.3.0, out-of-memory recovery is
+ automatic and this routine now does nothing.  THe interface is retained
+ to avoid link errors with legacy code.
 }
 
 api {} {
@@ -1173,7 +1488,7 @@ api {} {
 }
 
 api {} {
-  int sqlite3_db_handle(sqlite3_stmt*);
+  sqlite3 *sqlite3_db_handle(sqlite3_stmt*);
 } {
  Return the sqlite3* database handle to which the prepared statement given
  in the argument belongs.  This is the same database handle that was
@@ -1181,6 +1496,158 @@ api {} {
  the statement in the first place.
 }
 
+api {} {
+  void *sqlite3_update_hook(
+    sqlite3*, 
+    void(*)(void *,int ,char const *,char const *,sqlite_int64),
+    void*
+  );
+} {
+ Register a callback function with the database connection identified by the 
+ first argument to be invoked whenever a row is updated, inserted or deleted.
+ Any callback set by a previous call to this function for the same 
+ database connection is overridden.
+
+ The second argument is a pointer to the function to invoke when a 
+ row is updated, inserted or deleted. The first argument to the callback is
+ a copy of the third argument to sqlite3_update_hook. The second callback 
+ argument is one of SQLITE_INSERT, SQLITE_DELETE or SQLITE_UPDATE, depending
+ on the operation that caused the callback to be invoked. The third and 
+ fourth arguments to the callback contain pointers to the database and 
+ table name containing the affected row. The final callback parameter is 
+ the rowid of the row. In the case of an update, this is the rowid after 
+ the update takes place.
+
+ The update hook is not invoked when internal system tables are
+ modified (i.e. sqlite_master and sqlite_sequence).
+
+ If another function was previously registered, its pArg value is returned.
+ Otherwise NULL is returned.
+
+ See also: sqlite3_commit_hook(), sqlite3_rollback_hook()
+}
+
+api {} {
+  void *sqlite3_rollback_hook(sqlite3*, void(*)(void *), void*);
+} {
+ Register a callback to be invoked whenever a transaction is rolled
+ back. 
+
+ The new callback function overrides any existing rollback-hook
+ callback. If there was an existing callback, then it's pArg value 
+ (the third argument to sqlite3_rollback_hook() when it was registered) 
+ is returned. Otherwise, NULL is returned.
+
+ For the purposes of this API, a transaction is said to have been 
+ rolled back if an explicit "ROLLBACK" statement is executed, or
+ an error or constraint causes an implicit rollback to occur. The 
+ callback is not invoked if a transaction is automatically rolled
+ back because the database connection is closed.
+}
+
+api {} {
+  int sqlite3_enable_shared_cache(int);
+} {
+  This routine enables or disables the sharing of the database cache
+  and schema data structures between connections to the same database.
+  Sharing is enabled if the argument is true and disabled if the argument
+  is false.
+
+  Cache sharing is enabled and disabled on a thread-by-thread basis.
+  Each call to this routine enables or disables cache sharing only for
+  connections created in the same thread in which this routine is called.
+  There is no mechanism for sharing cache between database connections
+  running in different threads.
+
+  Sharing must be disabled prior to shutting down a thread or else
+  the thread will leak memory.  Call this routine with an argument of
+  0 to turn off sharing.  Or use the sqlite3_thread_cleanup() API.
+
+  This routine must not be called when any database connections
+  are active in the current thread.  Enabling or disabling shared
+  cache while there are active database connections will result
+  in memory corruption.
+
+  When the shared cache is enabled, the
+  following routines must always be called from the same thread:
+  sqlite3_open(), sqlite3_prepare(), sqlite3_step(), sqlite3_reset(),
+  sqlite3_finalize(), and sqlite3_close().
+  This is due to the fact that the shared cache makes use of
+  thread-specific storage so that it will be available for sharing
+  with other connections.
+
+  This routine returns SQLITE_OK if shared cache was
+  enabled or disabled successfully.  An error code is returned
+  otherwise.
+
+  Shared cache is disabled by default for backward compatibility.
+}
+
+api {} {
+  void sqlite3_thread_cleanup(void);
+} {
+  This routine makes sure that all thread local storage used by SQLite
+  in the current thread has been deallocated.  A thread can call this
+  routine prior to terminating in order to make sure there are no memory
+  leaks.
+
+  This routine is not strictly necessary.  If cache sharing has been
+  disabled using sqlite3_enable_shared_cache() and if all database
+  connections have been closed and if SQLITE_ENABLE_MEMORY_MANAGMENT is
+  on and all memory has been freed, then the thread local storage will
+  already have been automatically deallocated.  This routine is provided
+  as a convenience to the program who just wants to make sure that there
+  are no leaks.
+}
+
+api {} {
+  int sqlite3_release_memory(int N);
+} {
+  This routine attempts to free at least N bytes of memory from the caches
+  of database connecions that were created in the same thread from which this
+  routine is called.  The value returned is the number of bytes actually
+  freed.  
+
+  This routine is only available if memory management has been enabled
+  by compiling with the SQLITE_ENABLE_MEMORY_MANAGMENT macro.
+}
+
+api {} {
+  void sqlite3_soft_heap_limit(int N);
+} {
+  This routine sets the soft heap limit for the current thread to N.
+  If the total heap usage by SQLite in the current thread exceeds N,
+  then sqlite3_release_memory() is called to try to reduce the memory usage
+  below the soft limit.
+
+  Prior to shutting down a thread sqlite3_soft_heap_limit() must be set to 
+  zero (the default) or else the thread will leak memory. Alternatively, use
+  the sqlite3_thread_cleanup() API.
+
+  A negative or zero value for N means that there is no soft heap limit and
+  sqlite3_release_memory() will only be called when memory is exhaused.
+  The default value for the soft heap limit is zero.
+
+  SQLite makes a best effort to honor the soft heap limit.  But if it
+  is unable to reduce memory usage below the soft limit, execution will
+  continue without error or notification.  This is why the limit is 
+  called a "soft" limit.  It is advisory only.
+
+  This routine is only available if memory management has been enabled
+  by compiling with the SQLITE_ENABLE_MEMORY_MANAGMENT macro.
+}
+
+api {} {
+  void sqlite3_thread_cleanup(void);
+} {
+  This routine ensures that a thread that has used SQLite in the past
+  has released any thread-local storage it might have allocated.  
+  When the rest of the API is used properly, the cleanup of 
+  thread-local storage should be completely automatic.  You should
+  never really need to invoke this API.  But it is provided to you
+  as a precaution and as a potential work-around for future
+  thread-releated memory-leaks.
+}
 
 set n 0
 set i 0
@@ -1199,6 +1666,11 @@ foreach name [lsort [array names name_to_idx]] {
   set sname($i) $name
   incr i
 }
+#parray n_to_name
+#parray n_to_idx
+#parray name_to_idx
+#parray sname
+incr n -1
 puts {<table width="100%" cellpadding="5"><tr>}
 set nrow [expr {($n+2)/3}]
 set i 0
@@ -1243,6 +1715,9 @@ foreach name [lsort [array names name_to_idx]] {
   puts "</pre></blockquote>"
   regsub -all {\[} $desc {\[} desc
   regsub -all {sqlite3_[a-z0-9_]+} $desc "\[resolve_name $name &\]" d2
+  foreach x $specialname {
+    regsub -all $x $d2 "\[resolve_name $name &\]" d2
+  }
   regsub -all "\n( *\n)+" [subst $d2] "</p>\n\n<p>" d3
   puts "<p>$d3</p>"
 }
